/************************************************************************/
/* TaxSolve_OH_IT1040_2015.c - 						*/
/* Copyright (C) 2016 - Aston Roberts					*/
/* 									*/
/* Compile:   gcc taxsolve_OH_IT1040_2015.c -o taxsolve_OH_IT1040_2015	*/
/* Run:	      ./taxsolve_OH_IT1040_2015  OH_IT1040_2015.txt 		*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-7-2016	aston_roberts@yahoo.com			*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"

double thisversion=13.0;

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       1

double TaxRateFunction( double x, int status )
{
 if (x <  5200.0) return    x * 0.00495; else
 if (x < 10400.0) return    27.74 + (x-5200.0) 	 * 0.00990; else
 if (x < 15650.0) return    77.22 + (x-10400.0)  * 0.01980; else
 if (x < 20900.0) return   181.17 + (x-15650.0)  * 0.02476; else
 if (x < 41700.0) return   311.16 + (x-20900.0)  * 0.02969; else
 if (x < 83350.0) return   928.71 + (x-41700.0)  * 0.03465; else
 if (x < 104250.0) return 2371.88 + (x-83350.0)  * 0.03960; else
 if (x < 208500.0) return 3199.52 + (x-104250.0) * 0.04597;
 else 		   return 7991.89 + (x-208500.0) * 0.04997;
}



/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int j, k, mm;
 char word[1000], outfname[1000], label[50];
 int status=0, exemptions=0, qualify_jfc=0;
 time_t now;
 double factorA, factorB;
 double L2a, L2b, L7a, L8a, L8b, L8c;
 double jfc, exemption_amnt;
 double SchedA[MAX_LINES], SchedC[MAX_LINES];

 /* Intercept any command-line arguments. */
 printf("OH IT1040 2015 - v%3.1f\n", thisversion);
 mm = 1;  k=1;
 while (mm < argc)
 {
  if (strcmp(argv[mm],"-verbose")==0)  verbose = 1;
  else
  if (k==1)
   {
    infile = fopen(argv[mm],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[mm]);  exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[mm]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,"_out.txt"); else strcpy(&(outfname[j]),"_out.txt");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname);  exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else {printf("Unknown command-line parameter '%s'\n", argv[mm]); exit(1);}
  mm++;
 }

 if (infile==0) {printf("Error: No input file on command line.\n");  exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (mm=0; mm<MAX_LINES; mm++) 
  {
   L[mm] = 0.0;
   SchedA[mm] = 0.0;
   SchedC[mm] = 0.0;
  }

 /* Accept parameters from input file. */
 /* Expect  OH IT1040 lines, something like:
	Title:  OH IT1040 1999 Return
	L1		{Wages}
*/

 /* Accept Form's "Title" line, and put out with date-stamp for records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 /* get_parameter(infile, kind, x, emssg ) */
 get_parameter( infile, 's', word, "Status" );
 get_parameter( infile, 'l', word, "Status ?");
 if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 if (strncasecmp(word,"Married/Joint",11)==0) status = MARRIED_FILLING_JOINTLY; else
 if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD;
 else
  { 
   printf("Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house.\nExiting.\n", word); 
   fprintf(outfile,"Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house.\nExiting.\n", word); 
   exit(1); 
  }
 fprintf(outfile,"Status = %s (%d)\n", word, status);

 get_parameter( infile, 's', word, "Exemptions" );	/* Exemptions, self/depend. */
 get_parameters( infile, 'i', &exemptions, "Exemptions"); 

 /* Answer YES only if Married Filing Jointly, and you and your spouse */
 /* each have qualifying Ohio adjusted gross income of at least $500. */
 get_parameter( infile, 's', word, "JointCredit" );
 get_parameter( infile, 'b', &qualify_jfc, "JointCredit ?"); 

 GetLine( "L1", &L[1] );	/* Federal Adjusted Gross Income */
 GetLine( "L4", &L[4] );	/* Personal and dependent exemption deduction, Sched-J */
 GetLine( "L6", &L[6] );	/* Taxable business income (Ohio Schedule IT BUS, line 13) */
 GetLine( "L8b", &L8b );	/* Business income tax liability (Schedule IT BUS, line 14) */
 GetLine( "L11", &L[11] );	/* Interest penalty on underpayment of estimated tax (IT/SD 2210) */
 GetLine( "L12", &L[12] );	/* Sales and use tax due */
 GetLine( "L14", &L[14] );	/* Ohio Tax Withheld (box 17 on your W-2) */
 GetLine( "L15", &L[15] );	/* Estimated and extension payments made */
 GetLine( "L17", &L[17] );	/* Amended return only - amount previously paid with original */
 GetLine( "L19", &L[19] );	/* Amended return only - overpayment previously received on original */

 GetLine( "SchedA_1", &SchedA[1] );	/* Non-Ohio state or local gov't interest and dividends */
 GetLine( "SchedA_2", &SchedA[2] );	/* Ohio pass-through entity and financial institutions taxes paid */
 GetLine( "SchedA_3", &SchedA[3] );	/* Reimbursed college tuit. fees deducted prev yrs. */
 GetLine( "SchedA_4", &SchedA[4] );	/* Losses from sale or disposition of Ohio public obligations */
 GetLine( "SchedA_5", &SchedA[5] );	/* Nonmedical withdrawals from medical savings account */
 GetLine( "SchedA_6", &SchedA[6] );	/* Reimbursement of expenses previously deducted for Ohio income tax ...*/
 GetLine( "SchedA_7", &SchedA[7] );	/* Lump sum distribution add-back */
 GetLine( "SchedA_8", &SchedA[8] );	/* Adjustment for Internal Revenue Code sections 168(k) and 179 */
 GetLine( "SchedA_9", &SchedA[9] );	/* Federal interest and dividends subject to state taxation */
 GetLine( "SchedA_10", &SchedA[10] );	/* Miscellaneous federal income tax additions */

 GetLine( "SchedA_12", &SchedA[12] );	/* Business income deduction (Ohio Schedule IT BUS, line 11) */
 GetLine( "SchedA_13", &SchedA[13] );	/* Compensation earned in Ohio by residents of neighboring states */
 GetLine( "SchedA_14", &SchedA[14] );	/* State/municipal tax overpayments (IRS form 1040, line 10) */
 GetLine( "SchedA_15", &SchedA[15] );	/* Qualifying Social Security benefits */
 GetLine( "SchedA_16", &SchedA[16] );	/* Interest income from Ohio public obligations ... */
 GetLine( "SchedA_17", &SchedA[17] );	/* Amounts contributed to an individual development account */
 GetLine( "SchedA_18", &SchedA[18] );	/* Federal interest and dividends exempt from state taxation */
 GetLine( "SchedA_19", &SchedA[19] );	/* Adjustment for Internal Revenue Code 168(k), 179 depreciation */
 GetLine( "SchedA_20", &SchedA[20] );	/* Refund or reimbursements shown on IRS form 1040, line 21 */
 GetLine( "SchedA_21", &SchedA[21] );	/* Repayment of income reported in a prior year */
 GetLine( "SchedA_22", &SchedA[22] );	/* Wage expense not deducted ... */
 GetLine( "SchedA_23", &SchedA[23] );	/* Miscellaneous federal income tax deductions */

 GetLine( "SchedA_24", &SchedA[24] );	/* Military pay for Ohio residents received while stationed outside Ohio */
 GetLine( "SchedA_25", &SchedA[25] );	/* Income earned by military nonresidents ... */
 GetLine( "SchedA_26", &SchedA[26] );	/* Uniformed services retirement income */
 GetLine( "SchedA_27", &SchedA[27] );	/* Military injury relief fund */
 GetLine( "SchedA_28", &SchedA[28] );	/* Certain Ohio National Guard reimbursements and benefits */

 GetLine( "SchedA_29", &SchedA[29] );	/* Ohio 529 contributions, tuition credit purchases */
 GetLine( "SchedA_30", &SchedA[30] );	/* Pell College Opportunity taxable grant amounts used for room and board */

 GetLine( "SchedA_31", &SchedA[31] );	/* Disability and survivorship benefi ts */
 GetLine( "SchedA_32", &SchedA[32] );	/* Unreimbursed long-term care insurance premiums ... */
 GetLine( "SchedA_33", &SchedA[33] );	/* Funds deposited into, and earnings of, a medical savings account */
 GetLine( "SchedA_34", &SchedA[34] );	/* Qualified organ donor expenses */
 
 /* Schedule of Credits. */
 GetLine( "Credits_2", &SchedC[2] );	/* Retirement income credit */
 SchedC[2] = smallerof( SchedC[2], 200.0 );
 GetLine( "Credits_3", &SchedC[3] );	/* Lump sum retirement credit (Ohio LS WKS, line 6) */
 GetLine( "Credits_4", &SchedC[4] );	/* Senior citizen credit */
 GetLine( "Credits_5", &SchedC[5] );	/* Lump sum distribution credit */
 GetLine( "Credits_6", &SchedC[6] );	/* Child care and dependent care credit */

 GetLine( "Credits_8", &SchedC[8] );	/* Displaced worker training credit */
 GetLine( "Credits_9", &SchedC[9] );	/* Ohio political contributions credit */
 if (status == MARRIED_FILLING_JOINTLY)
  SchedC[9] = smallerof( SchedC[9], 100.0 );
 else
  SchedC[9] = smallerof( SchedC[9], 50.0 );

 GetLine( "Credits_14", &SchedC[14] );	/* Earned income credit */
 GetLine( "Credits_15", &SchedC[15] );	/* Ohio adoption credit */
 GetLine( "Credits_16", &SchedC[16] );	/* Job retention credit, nonrefundable portion */
 GetLine( "Credits_17", &SchedC[17] );	/* Credit for eligible new employees in an enterprise zone */
 GetLine( "Credits_18", &SchedC[18] );	/* Credit for certifi ed ethanol plant investments */
 GetLine( "Credits_19", &SchedC[19] );	/* Credit for purchases of grape production property */
 GetLine( "Credits_20", &SchedC[20] );	/* Credit for investing in an Ohio small business */
 GetLine( "Credits_21", &SchedC[21] );	/* Enterprise zone day care and training credits */
 GetLine( "Credits_22", &SchedC[22] );	/* Research and development credit */
 GetLine( "Credits_23", &SchedC[23] );	/* Ohio historic preservation credit, nonrefundable carryforward portion */

 GetLine( "Credits_26", &SchedC[26] );	/*  Portion L3 was not earned in Ohio. */

 GetLine( "Credits_29", &SchedC[29] );	/* Portion L3 taxed by other states */

 GetLine( "Credits_34", &SchedC[34] );	/* Manufacturing equipment grant */

 GetLine( "Credits_36", &SchedC[36] );	/* Historic preservation credit */
 GetLine( "Credits_37", &SchedC[37] );	/* Business jobs credit */
 GetLine( "Credits_38", &SchedC[38] );	/* Pass-through entity credit */
 GetLine( "Credits_39", &SchedC[39] );	/* Motion picture production credit */
 GetLine( "Credits_40", &SchedC[40] );	/* Financial Institutions Tax (FIT) credit */


 /* ---- Do Calculations. ---- */

 for (j=1; j <= 10; j++)
  SchedA[11] = SchedA[11] + SchedA[j];

 for (j=12; j <= 34; j++)
  SchedA[35] = SchedA[35] + SchedA[j];

 L2a = SchedA[11];
 L2b = SchedA[35];
 L[3] = L[1] + L2a + L2b;

 if (L[3] <= 40000.0)
  exemption_amnt = 2200.0;
 else
 if (L[3] <= 80000.0)
  exemption_amnt = 1950.0;
 else
  exemption_amnt = 1700.0;
 L[4] = exemption_amnt * exemptions;

 L[5] = NotLessThanZero( L[3] - L[4] );
 L[7] = NotLessThanZero( L[5] - L[6] );
 L7a = L[7];
 L8a = TaxRateFunction( L7a, status );
 L8c = L8a + L8b;

 SchedC[1] = L8c;	
 if (L[5] <= 10000)
  SchedC[7] = 88.0;
 else
  SchedC[7] = 0.0;	 
 if (L[5] < 30000)
  SchedC[10] = 20.0 * exemptions;
 
 for (j=2; j <= 10; j++)
  SchedC[11] = SchedC[11] + SchedC[j];		

 if ((status == MARRIED_FILLING_JOINTLY) && (qualify_jfc))
  { /*Joint_Filing_Credit*/
    if (L[5] < 25000) jfc = 0.20;
    else
    if (L[5] < 50000) jfc = 0.15;
    else
    if (L[5] < 75000) jfc = 0.10;
    else jfc = 0.05;
    SchedC[13] = smallerof( jfc * L[10], 650.0 );
  } /*Joint_Filing_Credit*/

 for (j=13; j <= 23; j++)
  SchedC[24] = SchedC[24] + SchedC[j];          
 SchedC[25] = NotLessThanZero( SchedC[12] - SchedC[24] );

 SchedC[27] = L[3];
 j = 10000.0 * SchedC[26] / SchedC[27];
 factorA = (double)j / 10000.0;
 // printf(" %4g\n", factorA );
 SchedC[28] = SchedC[25] * factorA;

 SchedC[30] = L[3];
 j = 10000.0 * SchedC[29] / SchedC[30];
 factorB = (double)j / 10000.0;
 // printf(" %4g\n", factorB );			
 SchedC[31] = SchedC[25] * factorB;
 SchedC[32] = L[13];
 SchedC[33] = smallerof( SchedC[31], SchedC[32] );

 SchedC[35] = SchedC[11] + SchedC[24] + SchedC[28] + SchedC[33] + SchedC[34];
 L[9] = SchedC[35];				

 for (j=36; j <= 40; j++)
  SchedC[41] = SchedC[41] + SchedC[j];          
 L[16] = SchedC[41];

 L[10] = NotLessThanZero( L8c - L[9] );
 L[13] = L[10] + L[11] + L[12];			/* Total Ohio tax liability before withholding or estimated payments. */
 L[18] = L[14] + L[15] + L[16] + L[17];		/* Total Ohio tax payments */
 L[20] = L[18] - L[19];
 if (L[13] >= L[20])
  {
   L[21] = L[13] - L[20];
   L[23] = L[21] + L[22];			/* TOTAL AMOUNT DUE */
  }
 else
  {
   L[24] = L[20] - L[13];			/* Overpayment */
   L[27] = L[24];
  }

 if ((status == SINGLE) && (L[1] <= 12200.0) && (SchedA[11] == 0.0))
  fprintf(outfile, "You do not need to file Ohio tax return (Fed AGI < minimum).\n");

 if ((status == MARRIED_FILLING_JOINTLY) && (L[1] <= 14400.0) && (SchedA[11] == 0.0))
  fprintf(outfile, "You do not need to file Ohio tax return (Fed AGI < minimum).\n");

 if ((L[4] >= L[3]) && (SchedA[11] == 0.0))
  fprintf(outfile, "You do not need to file Ohio tax return (L[4] >= L[3]).\n");


 /* Output the Results. */

 showline(1);
 showline_wlabel( "L2a", L2a );
 showline_wlabel( "L2b", L2b );
 for (j = 3; j <= 12; j++)
  showline(j);
 showline_wmsg( 13, "Total Ohio tax liability" );
 showline_wmsg( 14, "Ohio income tax withheld" );
 for (j = 15; j <= 17; j++)
  showline(j);
 showline_wmsg( 18, "Total Ohio tax payments" );
 for (j = 19; j <= 20; j++)
  showline(j);
 if (L[13] >= L[20])
  {
    showline(21);
    showline(22);
    showline_wmsg( 23, "TOTAL AMOUNT DUE !!!" );
  }
 else
  {
    showline_wmsg( 24, "Overpayment" );
    showline_wmsg( 27, "YOUR REFUND !!!" );
  }

 fprintf(outfile,"\n-- 2015 Ohio Schedule A --\n");
 for (j = 1; j <= 35; j++)
  {
   sprintf( label, "SchedA%d", j );
   showline_wlabel( label, SchedA[j] );
  }

 fprintf(outfile,"\n-- 2015 Ohio Schedule of Credits --\n");
 for (j = 1; j <= 27; j++)
  {
   sprintf( label, "Credits%d", j );
   showline_wlabel( label, SchedC[j] );
  }
 fprintf(outfile,"   (Credit28 Factor [%g])\n", factorA );
 showline_wlabel( "Credits28", SchedC[28] );
 showline_wlabel( "Credits29", SchedC[29] );
 showline_wlabel( "Credits30", SchedC[30] );
 fprintf(outfile,"   (Credit31 Factor [%g])\n", factorB );

 for (j = 31; j <= 41; j++)
  {
   sprintf( label, "Credits%d", j );
   showline_wlabel( label, SchedC[j] );
  }

 fclose(infile);
 fclose(outfile);
 Display_File( outfname );
 printf("\nResults written to file:  %s\n", outfname);
 return 0;
}
